SuspendedGenerator

Struct SuspendedGenerator 

pub struct SuspendedGenerator<V, R> { /* private fields */ }
A suspended generator.

A suspended generator is like a normal generator with the minimal vertex moved out of the generator. This state results when a TryRamify implementation returns an error.

Iteration can be resumed, which requires specifying an iterator over children manually.

Implementations

Source

impl<V, R> SuspendedGenerator<V, R>

pub fn resume<I, W: DiagramWrite>( self, writer: &mut W, children: I, ) -> Result<Generator<V, R>, W::Error>
where R: TryRamify<V>, I: IntoIterator<Item = V>,

Recover from an error and resume iteration.

This writes the current minimal vertex and updates the internal state to hold the provided children. The end result is equivalent to the TryRamify implementation succeeding and yielding the provided children.

pub fn into_active_vertices(self) -> impl ExactSizeIterator<Item = V>

Consume the suspended generator, returning an iterator over the active vertices in an unspecified order.

pub fn peek_annotation(&self) -> &str

Returns the annotation that will be written if iteration is resumed.