Charset

Struct Charset 

pub struct Charset {
    pub space: char,
    pub down_and_left: char,
    pub down_and_right: char,
    pub up_and_left: char,
    pub up_and_right: char,
    pub horizontal: char,
    pub vertical: char,
    pub down_and_horizontal: char,
    pub up_and_horizontal: char,
    pub vertical_and_left: char,
    pub vertical_and_right: char,
    pub vertical_and_horizontal: char,
}
A set of characters to use in the diagram.

In most cases, you would use some combination of box-drawing characters.

The rounded_corners character set often looks the best if there is font support, whereas the sharp_corners tends to have more universal font support.

When used in the Style struct, an IOWriter or a FmtWriter expects that these are characters with width exactly 1 when displayed to the terminal. Other characters, such as control characters or double-width characters (mainly those described in Unicode Annex #11) will corrupt the branch diagram.

Fields

space: char

The character.

down_and_left: char

The character.

down_and_right: char

The character.

up_and_left: char

The character.

up_and_right: char

The character.

horizontal: char

The character.

vertical: char

The character.

down_and_horizontal: char

The character.

up_and_horizontal: char

The character.

vertical_and_left: char

The character.

vertical_and_right: char

The character.

vertical_and_horizontal: char

The character.

Implementations

Source

impl Charset

pub const fn style(self) -> Style

Using this character set in a diagram style.

pub const fn rounded_corners() -> Self

The rounded corners character set.

╯ ┴ ╰
┤ ┼ ├ ─
╮ ┬ ╭ │
pub const fn sharp_corners() -> Self

The sharp corners character set.

┘ ┴ └
┤ ┼ ├ ─
┐ ┬ ┌ │
pub const fn doubled_lines() -> Self

The doubled lines character set.

╝ ╩ ╚
╣ ╬ ╠ ═
╗ ╦ ╔ ║
pub const fn invert(self) -> Self

Flip the orientation of the charset vertically.

This swaps the down_* characters with the up_* characters. This is useful when you want to render the branch diagram with the root at the bottom since it will make the branch diagram lines connect correctly when the lines are written in reverse order.

Also see Config::inverted_annotations to modify the layout algorithm to draw annotations correctly.