pub struct SuspendedGenerator<V, R> { /* private fields */ }Expand description
Implementations
Sourceimpl<V, R> SuspendedGenerator<V, R>
impl<V, R> SuspendedGenerator<V, R>
Sourcepub fn resume<I, W: DiagramWrite>(
self,
writer: &mut W,
children: I,
) -> Result<Generator<V, R>, W::Error>where
R: TryRamify<V>,
I: IntoIterator<Item = V>,
pub fn resume<I, W: DiagramWrite>(
self,
writer: &mut W,
children: I,
) -> Result<Generator<V, R>, W::Error>where
R: TryRamify<V>,
I: IntoIterator<Item = V>,
Recover from an error and resume iteration.
This writes the current minimal vertex and updates the internal state to hold the provided children.
The end result is equivalent to the TryRamify implementation succeeding and yielding
the provided children.
Sourcepub fn into_active_vertices(self) -> impl ExactSizeIterator<Item = V>
pub fn into_active_vertices(self) -> impl ExactSizeIterator<Item = V>
Consume the suspended generator, returning an iterator over the active vertices in an unspecified order.
Sourcepub fn peek_annotation(&self) -> &str
pub fn peek_annotation(&self) -> &str
Returns the annotation that will be written if iteration is resumed.